﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using BenchmarkDotNet.Attributes;
using BenchmarkDotNet.Running;

namespace ConsoleApp29
{
    class Program
    {
        static void Main()
        {
            BenchmarkRunner.Run<EnumToStringBenchmark>();
        }
    }

    public enum Color
    {
        AliceBlue,
        AntiqueWhite,
        Aqua,
        Aquamarine,
        Azure,
    }

    [RyuJitX64Job] // Run with x64 and x86 runtimes
    [MemoryDiagnoser] // Analyse the memory usage
    public partial  class EnumToStringBenchmark
    {
    }


    public static class EnumExtensions
    {
        private static readonly Dictionary<Enum, string> s_cache = new Dictionary<Enum, string>();
        private static readonly ConcurrentDictionary<Enum, string> s_cacheConcurrent = new ConcurrentDictionary<Enum, string>();

        public static string ToStringCached(this Enum value)
        {
            if (s_cache.TryGetValue(value, out var result))
                return result;

            return s_cache[value] = value.ToString();
        }

        public static string ToStringCachedConcurrent(this Enum value)
        {
            return s_cacheConcurrent.GetOrAdd(value, v => v.ToString());
        }

        private static readonly string[] s_cacheArray = GetEnumStrings();

        private static string[] GetEnumStrings()
        {
            System.Collections.IList list = Enum.GetValues(typeof(Color));

            var result = new string[list.Count];
            for (int i = 0; i < list.Count; i++)
            {
                result[i] = list[i].ToString();
            }

            return result;
        }

        public static string ToStringCachedArray(this Color myEnum)
        {
            return s_cacheArray[(int)myEnum];
        }

        public static string ToStringSwitch(this Color myEnum)
        {
            return myEnum switch
            {
                Color.AliceBlue => nameof(Color.AliceBlue),
                Color.AntiqueWhite => nameof(Color.AntiqueWhite),
                Color.Aqua => nameof(Color.Aqua),
                Color.Aquamarine => nameof(Color.Aquamarine),
                Color.Azure => nameof(Color.Azure),
                _ => throw new NotImplementedException(),
            };
        }
    }
}
